@@ -347,6 +347,56 @@ class WithTest(CoverageTest):
             branchz_missing="",
         )
 
+    def test_multiline_with(self) -> None:
+        # https://github.com/nedbat/coveragepy/issues/1880
+        self.check_coverage("""\
+            import contextlib, itertools
+            nums = itertools.count()
+            with (
+                contextlib.nullcontext() as x,
+            ):
+                while next(nums) < 6:
+                    y = 7
+            z = 8
+            """,
+            branchz="67 68",
+            branchz_missing="",
+        )
+
+    def test_multi_multiline_with(self) -> None:
+        # https://github.com/nedbat/coveragepy/issues/1880
+        self.check_coverage("""\
+            import contextlib, itertools
+            nums = itertools.count()
+            with (
+                contextlib.nullcontext() as x,
+                contextlib.nullcontext() as y,
+                contextlib.nullcontext() as z,
+            ):
+                while next(nums) < 8:
+                    y = 9
+            z = 10
+            """,
+            branchz="89 8A",
+            branchz_missing="",
+        )
+
+    def test_multi_multiline_with_backslash(self) -> None:
+        # https://github.com/nedbat/coveragepy/issues/1880
+        self.check_coverage("""\
+            import contextlib, itertools
+            nums = itertools.count()
+            with contextlib.nullcontext() as x, \\
+                contextlib.nullcontext() as y, \\
+                contextlib.nullcontext() as z:
+                while next(nums) < 6:
+                    y = 7
+            z = 8
+            """,
+            branchz="67 68",
+            branchz_missing="",
+        )
+
 
 class LoopArcTest(CoverageTest):
     """Arc-measuring tests involving loops."""
@@ -435,8 +485,6 @@ class LoopArcTest(CoverageTest):
         )
 
     def test_while_true(self) -> None:
-        # With "while True", 2.x thinks it's computation,
-        # 3.x thinks it's constant.
         self.check_coverage("""\
             a, i = 1, 0
             while True:
@@ -450,6 +498,20 @@ class LoopArcTest(CoverageTest):
             branchz_missing="",
         )
 
+    def test_while_not_false(self) -> None:
+        self.check_coverage("""\
+            a, i = 1, 0
+            while not False:
+                if i >= 3:
+                    a = 4
+                    break
+                i += 1
+            assert a == 4 and i == 3
+            """,
+            branchz="34 36",
+            branchz_missing="",
+        )
+
     def test_zero_coverage_while_loop(self) -> None:
         # https://github.com/nedbat/coveragepy/issues/502
         self.make_file("main.py", "print('done')")
@@ -485,6 +547,18 @@ class LoopArcTest(CoverageTest):
             branchz_missing="",
         )
 
+    def test_missing_while_body(self) -> None:
+        self.check_coverage("""\
+            a = 3; b = 0
+            if 0:
+                while a > 0:
+                    a -= 1
+            assert a == 3 and b == 0
+            """,
+            branchz="",
+            branchz_missing="",
+        )
+
     def test_for_if_else_for(self) -> None:
         self.check_coverage("""\
             def branches_2(l):
@@ -1843,7 +1917,7 @@ class LambdaArcTest(CoverageTest):
 # I don't understand why it failed, I don't understand why it passed,
 # so just skip the whole thing.
 skip_eventlet_670 = pytest.mark.skipif(
-    env.PYVERSION[:2] == (3, 9) and env.CPYTHON and env.OSX,
+    env.PYVERSION[:2] == (3, 9) and env.CPYTHON and env.MACOS,
     reason="Avoid an eventlet bug on Mac 3.9: eventlet#670",
     # https://github.com/eventlet/eventlet/issues/670
 )
